@@ -2,11 +2,12 @@ import logging
 import os
 from weakref import ref
 
-from ..config import config
-from ..utils import format_bytes, PeriodicCallback, log_errors, ignoring
+import dask
 
 from .adaptive import Adaptive
 
+from ..utils import format_bytes, PeriodicCallback, log_errors, ignoring
+
 logger = logging.getLogger(__name__)
 
 
@@ -92,7 +93,11 @@ class Cluster(object):
             if n >= len(self.scheduler.workers):
                 self.scheduler.loop.add_callback(self.scale_up, n)
             else:
-                to_close = self.scheduler.workers_to_close(n=len(self.scheduler.workers) - n)
+                to_close = self.scheduler.retire_workers(
+                    remove=False,
+                    close_workers=True,
+                    n=len(self.scheduler.workers) - n
+                )
                 logger.debug("Closing workers: %s", to_close)
                 self.scheduler.loop.add_callback(self.scale_down, to_close)
 
@@ -137,7 +142,7 @@ class Cluster(object):
         layout = Layout(width='150px')
 
         if 'bokeh' in self.scheduler.services:
-            template = config.get('diagnostics-link', 'http://{host}:{port}/status')
+            template = dask.config.get('distributed.dashboard.link')
 
             host = self.scheduler.address.split('://')[1].split(':')[0]
             port = self.scheduler.services['bokeh'].port
