@@ -8,6 +8,7 @@ from .serialize import (
     serialize, deserialize, nested_deserialize, Serialize, Serialized,
     to_serialize, register_serialization, register_serialization_lazy,
     serialize_bytes, deserialize_bytes, serialize_bytelist,
+    register_serialization_family,
 )
 
 from ..utils import ignoring
@@ -36,3 +37,8 @@ def _register_keras():
 @partial(register_serialization_lazy, "sparse")
 def _register_sparse():
     from . import sparse
+
+
+@partial(register_serialization_lazy, "pyarrow")
+def _register_arrow():
+    from . import arrow
